@@ -48,11 +48,11 @@
 OSDefineMetaClassAndAbstractStructors(IOAudioEngine, IOService)
 
 OSMetaClassDefineReservedUsed(IOAudioEngine, 0);
+OSMetaClassDefineReservedUsed(IOAudioEngine, 1);
+OSMetaClassDefineReservedUsed(IOAudioEngine, 2);
+OSMetaClassDefineReservedUsed(IOAudioEngine, 3);
+OSMetaClassDefineReservedUsed(IOAudioEngine, 4);
 
-OSMetaClassDefineReservedUnused(IOAudioEngine, 1);
-OSMetaClassDefineReservedUnused(IOAudioEngine, 2);
-OSMetaClassDefineReservedUnused(IOAudioEngine, 3);
-OSMetaClassDefineReservedUnused(IOAudioEngine, 4);
 OSMetaClassDefineReservedUnused(IOAudioEngine, 5);
 OSMetaClassDefineReservedUnused(IOAudioEngine, 6);
 OSMetaClassDefineReservedUnused(IOAudioEngine, 7);
@@ -107,6 +107,35 @@ IOReturn IOAudioEngine::performFormatChange(IOAudioStream *audioStream, const IO
     return kIOReturnUnsupported;
 }
 
+IOBufferMemoryDescriptor *IOAudioEngine::getStatusDescriptor()
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioEngine[%p]::getStatusDescriptor()\n", this);
+#endif
+	assert(reserved);
+
+	return reserved->statusDescriptor;
+}
+
+IOReturn IOAudioEngine::getNearestStartTime(IOAudioStream *audioStream, IOAudioTimeStamp *ioTimeStamp, bool isInput)
+{
+	return kIOReturnSuccess;
+}
+
+IOBufferMemoryDescriptor * IOAudioEngine::getBytesInInputBufferArrayDescriptor()
+{
+	assert(reserved);
+
+	return reserved->bytesInInputBufferArrayDescriptor;
+}
+
+IOBufferMemoryDescriptor * IOAudioEngine::getBytesInOutputBufferArrayDescriptor()
+{
+	assert(reserved);
+
+	return reserved->bytesInOutputBufferArrayDescriptor;
+}
+
 // Original code from here forward:
 SInt32 compareAudioStreams(IOAudioStream *stream1, IOAudioStream *stream2, void *ref)
 {
@@ -223,7 +252,19 @@ bool IOAudioEngine::init(OSDictionary *properties)
     
     numActiveUserClients = 0;
 
-    status = (IOAudioEngineStatus *)IOMallocAligned(round_page_32(sizeof(IOAudioEngineStatus)), PAGE_SIZE);
+	reserved = (ExpansionData *)IOMalloc (sizeof(struct ExpansionData));
+	if (!reserved) {
+		return false;
+	}
+	reserved->bytesInInputBufferArrayDescriptor = NULL;
+	reserved->bytesInOutputBufferArrayDescriptor = NULL;
+
+	reserved->statusDescriptor = IOBufferMemoryDescriptor::withOptions(kIODirectionOutIn | kIOMemoryKernelUserShared, round_page_32(sizeof(IOAudioEngineStatus)), page_size);
+//	reserved->statusDescriptor = IOBufferMemoryDescriptor::withOptions(kIODirectionOutIn | kIOMemoryKernelUserShared, round_page(sizeof(IOAudioEngineStatus)), page_size);
+	if (!reserved->statusDescriptor) {
+		return false;
+	}
+	status = (IOAudioEngineStatus *)reserved->statusDescriptor->getBytesNoCopy();
 
     if (!status) {
         return false;
@@ -249,11 +290,17 @@ bool IOAudioEngine::init(OSDictionary *properties)
         return false;
     }
     
-    bzero(status, round_page_32(sizeof(IOAudioEngineStatus)));
+	bzero(status, round_page_32(sizeof(IOAudioEngineStatus)));
     status->fVersion = kIOAudioEngineCurrentStatusStructVersion;
 
     setState(kIOAudioEngineStopped);
 
+#if __i386__
+	setProperty(kIOAudioEngineFlavorKey, (UInt32)kIOAudioStreamByteOrderLittleEndian, sizeof(UInt32)*8);
+#elif __ppc__
+	setProperty(kIOAudioEngineFlavorKey, (unsigned long long)kIOAudioStreamByteOrderBigEndian, sizeof(UInt32)*8);
+#endif
+
     return true;
 }
 
@@ -263,10 +310,11 @@ void IOAudioEngine::free()
     IOLog("IOAudioEngine[%p]::free()\n", this);
 #endif
 
-    if (status) {
-        IOFreeAligned(status, round_page_32(sizeof(IOAudioEngineStatus)));
-        status = 0;
-    }
+	if (reserved->statusDescriptor) {
+		reserved->statusDescriptor->release();
+		reserved->statusDescriptor = NULL;
+		status = NULL;
+	}
 
     if (outputStreams) {
         outputStreams->release();
@@ -306,7 +354,7 @@ void IOAudioEngine::free()
 	if (reserved) {
 		IOFree (reserved, sizeof(struct ExpansionData));
 	}
-    
+
     super::free();
 }
 
@@ -478,9 +526,10 @@ OSString *IOAudioEngine::getGlobalUniqueID()
     }
         
     uniqueIDStr = (char *)IOMallocAligned(uniqueIDSize, sizeof (char));
-    bzero(uniqueIDStr, uniqueIDSize);
     
     if (uniqueIDStr) {
+		bzero(uniqueIDStr, uniqueIDSize);
+
         if (className) {
             sprintf(uniqueIDStr, "%s:", className);
         }
@@ -913,6 +962,11 @@ IOReturn IOAudioEngine::addAudioStream(IOAudioStream *stream)
                 if (outputStreams->getCount() == 1) {
                     setRunEraseHead(true);
                 }
+
+				if (reserved->bytesInOutputBufferArrayDescriptor) {
+					reserved->bytesInOutputBufferArrayDescriptor->release();
+				}
+				reserved->bytesInOutputBufferArrayDescriptor = IOBufferMemoryDescriptor::withOptions(kIODirectionOutIn | kIOMemoryKernelUserShared, round_page(outputStreams->getCount() * sizeof(UInt32)), page_size);
                 break;
             case kIOAudioStreamDirectionInput:
                 assert(inputStreams);
@@ -920,6 +974,11 @@ IOReturn IOAudioEngine::addAudioStream(IOAudioStream *stream)
                 inputStreams->setObject(stream);
                 
                 maxNumInputChannels += stream->getMaxNumChannels();
+
+				if (reserved->bytesInInputBufferArrayDescriptor) {
+					reserved->bytesInInputBufferArrayDescriptor->release();
+				}
+				reserved->bytesInInputBufferArrayDescriptor = IOBufferMemoryDescriptor::withOptions(kIODirectionOutIn | kIOMemoryKernelUserShared, round_page(inputStreams->getCount() * sizeof(UInt32)), page_size);
                 break;
         }
 
@@ -953,6 +1012,10 @@ void IOAudioEngine::detachAudioStreams()
             iterator->release();
         }
         outputStreams->flushCollection();
+		if (reserved->bytesInOutputBufferArrayDescriptor) {
+			reserved->bytesInOutputBufferArrayDescriptor->release();
+			reserved->bytesInOutputBufferArrayDescriptor = NULL;
+		}
     }
     
     if (inputStreams && (inputStreams->getCount() > 0)) {
@@ -966,6 +1029,10 @@ void IOAudioEngine::detachAudioStreams()
             iterator->release();
         }
         inputStreams->flushCollection();
+		if (reserved->bytesInInputBufferArrayDescriptor) {
+			reserved->bytesInInputBufferArrayDescriptor->release();
+			reserved->bytesInInputBufferArrayDescriptor = NULL;
+		}
     }
 }
 
@@ -1183,18 +1250,18 @@ void IOAudioEngine::updateChannelNumbers()
                 if (channelID != 0) {
                     switch (control->getUsage()) {
                         case kIOAudioControlUsageOutput:
-                            if (outputChannelNumbers && (channelID <= maxNumOutputChannels)) {
-                                control->setChannelNumber(outputChannelNumbers[channelID - 1]);
-                            } else {
-                                control->setChannelNumber(kIOAudioControlChannelNumberInactive);
-                            }
+								if (outputChannelNumbers && (channelID <= maxNumOutputChannels)) {
+									control->setChannelNumber(outputChannelNumbers[channelID - 1]);
+								} else {
+									control->setChannelNumber(kIOAudioControlChannelNumberInactive);
+							}
                             break;
                         case kIOAudioControlUsageInput:
-                            if (inputChannelNumbers && (channelID <= maxNumInputChannels)) {
-                                control->setChannelNumber(inputChannelNumbers[channelID - 1]);
-                            } else {
-                                control->setChannelNumber(kIOAudioControlChannelNumberInactive);
-                            }
+							if (inputChannelNumbers && (channelID <= maxNumInputChannels)) {
+								control->setChannelNumber(inputChannelNumbers[channelID - 1]);
+							} else {
+								control->setChannelNumber(kIOAudioControlChannelNumberInactive);
+							}
                             break;
                         case kIOAudioControlUsagePassThru:
                             if (inputChannelNumbers) {
@@ -1553,7 +1620,25 @@ AbsoluteTime IOAudioEngine::getTimerInterval()
     } else if ((numErasesPerBuffer == 0) || (!getRunEraseHead())) {	// Run once per ring buffer
         nanoseconds_to_absolutetime(((UInt64)NSEC_PER_SEC * (UInt64)getNumSampleFramesPerBuffer() / (UInt64)currentRate->whole), &interval);
     } else {
-        nanoseconds_to_absolutetime(((UInt64)NSEC_PER_SEC * (UInt64)getNumSampleFramesPerBuffer() / (UInt64)currentRate->whole / (UInt64)numErasesPerBuffer), &interval);
+        OSCollectionIterator *outputIterator;
+        IOAudioStream *outputStream;
+		UInt32 bufferSize;
+		UInt32 newNumErasesPerBuffer;
+
+		outputIterator = OSCollectionIterator::withCollection(outputStreams);
+
+		if (outputIterator) {
+			while (outputStream = (IOAudioStream *)outputIterator->getNextObject()) {
+				bufferSize = outputStream->getSampleBufferSize();
+				if ((bufferSize / numErasesPerBuffer) > 65536) {
+					newNumErasesPerBuffer = bufferSize / 65536;
+					if (newNumErasesPerBuffer > numErasesPerBuffer) {
+						numErasesPerBuffer = newNumErasesPerBuffer;
+					}
+				}
+			}
+		}
+		nanoseconds_to_absolutetime(((UInt64)NSEC_PER_SEC * (UInt64)getNumSampleFramesPerBuffer() / (UInt64)currentRate->whole / (UInt64)numErasesPerBuffer), &interval);
     }
     return interval;
 }
